% Risk Neutral Distribution estimation using a mixture of lognormals.
% Uses Bank of America (BAC) options and a mixture of 3 lognormals.
clc; clear;

%% Bank of America data
% Hard code BAC spot and interest rate
S = 16.18;
r = 0.01;
q = 0;

% BAC read in maturity, calls, puts, and strikes.
T = 177/365;
Calls = [...
  6.35 5.60 4.75 3.95 3.25 2.59 2.01 1.51 1.10 0.77 0.53 0.35 0.21 0.18;
  6.60 5.70 4.85 4.05 3.35 2.65 2.06 1.55 1.14 0.81 0.55 0.36 0.24 0.20]';
Puts = [...
  0.33 0.45 0.61 0.81 1.07 1.40 1.81 2.31 2.89 3.50 4.30 5.10 5.95 6.40;
  0.36 0.48 0.63 0.84 1.10 1.44 1.85 2.35 2.93 3.60 4.40 5.20 6.15 6.55]';
K = [10 11 12 13 14 15 16 17 18 19 20 21 22 22.5];

% Define the market price as the average of bids and asks.
Call = (Calls(:,1) + Calls(:,2))./2;
Put  = (Puts(:,1)  + Puts(:,2)) ./2;

%% Extract the implied volatilities
for k=1:length(Call)
    CallIV(k) = fminsearch(@(v) BSIV(v,Call(k),S,K(k),r,q,T,'C'), 0.15);
    PutIV(k)  = fminsearch(@(v) BSIV(v,Put(k), S,K(k),r,q,T,'P'), 0.15);
end
subplot(2,2,1)
plot(K,CallIV,'kx-',K,PutIV,'rx-')
legend('Calls','Puts')
title('Market Implied Vol')


%% Parameter estimation
% Each lognormal requires 3 parameters: mean, stdev, and weight
% The last set of parameters requires no weight -- it's defined as the
% completement of the others.
% The algorithm estimates these parameters, and so requires starting values.
% In this example we use 3 lognormals.  Usually 2 or 3 is sufficient.
LNmeans  = [2.0 3.0 3.0];
LNstdev  = [0.5 0.2 0.3];
LNweight = [0.2 0.2];

% Starting values fo the optimization.
start = [LNmeans LNstdev LNweight];

% Esimate the parameters.
options = optimset('MaxFunEvals', 1e5, 'MaxIter', 1e5);
beta = fminsearch(@(b) FindLNparams(b,Call,Put,S,K,r,T), start, options);

% M = number of mixtures
M = (length(beta)+1)/3;

% Return the estimated LN means and std deviations.
m = beta(1:M);
s = beta(M+1:2*M);

% Return the M-1 weights and construct the last weight as the complement of
% the others.
p = beta(2*M+1:end);
p(M) = 1 - sum(p(1:M-1));

%% Output the estimation results
clc;
fprintf('Result of lognormal mixture parameter estimation \n')
fprintf('Member  LN mean    LN std     Weight \n')
fprintf('--------------------------------------- \n')
for i=1:M
    fprintf(' %2.0f   %8.4f   %8.4f   %8.4f \n',i,m(i),s(i),p(i));
end
fprintf('--------------------------------------- \n')


%% Generate the mixed lognormal RND and plot.
ST = [1:0.01:40];
for i=1:length(ST);
	f(i) = LNmixdensity(ST(i),m,s,p);
end

subplot(2,2,2)
plot(ST,f)
title('Risk Neutral Density')

%% Generate the lognormal mixture prices and implied vols
for k=1:length(Call)
    LNCall(k) = LNmixPrice(K(k),S,m,s,p,r,T,'C');
    LNPut(k)  = LNmixPrice(K(k),S,m,s,p,r,T,'P');
    LNCallIV(k) = fminsearch(@(v) BSIV(v,LNCall(k),S,K(k),r,q,T,'C'), 0.15);
    LNPutIV(k)  = fminsearch(@(v) BSIV(v,LNPut(k), S,K(k),r,q,T,'P'), 0.15);
end

subplot(2,2,3)
plot(K,CallIV,'kx-',K,LNCallIV,'rx-')
legend('Market','LN-mix')
title('Call Implied Vol')

subplot(2,2,4)
plot(K,PutIV,'kx-',K,LNPutIV,'rx-')
legend('Market','LN-mix')
title('Put Implied Vol')

