% Risk Neutral Distribution estimation using a mixture of lognormals.
% Uses Bank of America (BAC) options and a mixture of 3 lognormals.
clc; clear;

% Anonymous functions for the Black Scholes call and put
BSCall = @(s,K,r,q,v,T) s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) - v*sqrt(T));
BSPut =  @(s,K,r,q,v,T) K*exp(-r*T)*normcdf(-(log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T) + v*sqrt(T)) - s*exp(-q*T)*normcdf(-(log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T));

%% Data for BAC
% BAC spot and interest rate
S = 16.18;
rf = 0.01;

% BAC read in maturity, calls, puts, and strikes.
T = 177/365;
Calls = [...
  6.35 5.60 4.75 3.95 3.25 2.59 2.01 1.51 1.10 0.77 0.53 0.35 0.21 0.18;
  6.60 5.70 4.85 4.05 3.35 2.65 2.06 1.55 1.14 0.81 0.55 0.36 0.24 0.20]';
Puts = [...
  0.33 0.45 0.61 0.81 1.07 1.40 1.81 2.31 2.89 3.50 4.30 5.10 5.95 6.40;
  0.36 0.48 0.63 0.84 1.10 1.44 1.85 2.35 2.93 3.60 4.40 5.20 6.15 6.55]';
K = [10 11 12 13 14 15 16 17 18 19 20 21 22 22.5];

% Define the market price as the average of bids and asks.
Call = (Calls(:,1) + Calls(:,2))./2;
Put  = (Puts(:,1)  + Puts(:,2)) ./2;

%% Lognormal mixture parameter estimation
% Each lognormal requires 3 parameters: mean, stdev, and weight
% The last set of parameters requires no weight -- it's defined as the
% completement of the others.
% The algorithm estimates these parameters, and so requires starting values.
% In this example we use 2 lognormals.  Usually 2 or 3 is sufficient.
LNmeans  = [2.0 3.0];
LNstdev  = [0.5 0.2];
LNweight = [0.5];

% LNmeans  = [2.0 3.0 3.0];
% LNstdev  = [0.5 0.2 0.3];
% LNweight = [0.2 0.2];


% Starting values fo the optimization.
start = [LNmeans LNstdev LNweight];

% Estimate the parameters.
options = optimset('MaxFunEvals', 1e5, 'MaxIter', 1e5);
beta = fminsearch(@(b) FindLNparams(b,Call,Put,S,K,rf,T), start, options);

% M = number of members
M = (length(beta)+1)/3;

% Return the estimated LN means and std deviations.
m = beta(1:M);
s = beta(M+1:2*M);

% Return the M-1 weights and construct the last weight as the complement of
% the others.
p = beta(2*M+1:end);
p(M) = 1 - sum(p(1:M-1));

%% Display the estimation results
clc;
fprintf('Result of lognormal mixture parameter estimation \n')
fprintf('Member  LN mean    LN std     Weight \n')
fprintf('--------------------------------------- \n')
for i=1:M
    fprintf(' %2.0f   %8.4f   %8.4f   %8.4f \n',i,m(i),s(i),p(i));
end
fprintf('--------------------------------------- \n')
fprintf('\n')


%% Generate the mixed lognormal RND
dS = 0.01;
ST = [0.0001:dS:35];
for i=1:length(ST);
	LN_RND(i) = LNmixdensity(ST(i),m,s,p);
end

% Calculate area under the RND
AreaUnderLN_RND  = trapz(ST(1:length(LN_RND)), LN_RND);

%% Obtain the original implied volatilities
start = 0.4;
for i=1:length(Call)
	IVC(i) = fminsearch(@(v) BSIV(v,Call(i),S,K(i),rf,0,T,'C'), start);
	IVP(i) = fminsearch(@(v) BSIV(v,Put(i),S,K(i),rf,0,T,'P'), start);
	IV(i) = (IVC(i) + IVP(i))/2;
end

% Fit the quadratic model to the IV.
beta = regstats(IV,K,'purequadratic');
B = beta.beta;

% Define the expanded strike range
dK = 0.01;
K2 = [1:dK:35];

% Obtain fitted IV and call prices along expanded strike range.
for i=1:length(K2);
	Vol(i) = B(1) + B(2)*K2(i) + B(3)*K2(i)^2;
	FittedCall(i) = BSCall(S,K2(i),rf,0,Vol(i),T);
end

% Plot the quadratic fitted IV and the market IV
subplot(1,3,1)
plot(K2,Vol,K,IV,'x')
legend('Quadratic DVF', 'Market')
title('Implied Volatilities')

%% Extract the RND from the fitted call prices
[DVF_RND, K3] = ExtractRND(K2,FittedCall,'cubic');

% Calculate area under the RND
AreaUnderDVF_RND  = trapz(ST(1:length(DVF_RND)), DVF_RND);

% Plot the two RND
subplot(1,3,2)
plot(K3,DVF_RND,'g-',ST,LN_RND,'r-')
legend('Quadratic DVF', 'LN Mix')
title('Risk Neutral Densities')

%% Recover the call prices
K5 = ST;
dK = dS;
for k=1:length(K);
    Kstar = K(k);
	% Recover the call prices using the Lognormal RND
    I = find(K5<=Kstar);
	NewStrike = K5;
	NewStrike(I) = 0;
    Integrand = max(NewStrike - Kstar,0).*LN_RND;
    LNRecoveredCall(k)  = exp(-rf*T)*trapz(Integrand).*dK;
	LNPercentError(k)  = (LNRecoveredCall(k) - Call(k))/Call(k)*100;
	clear I
	% Recover the call prices using the DVF quadratic RND
    I = find(K3<=Kstar);
	NewStrike = K3;
	NewStrike(I) = 0;
    Integrand = max(NewStrike - Kstar,0).*DVF_RND;
    DVFRecoveredCall(k)  = exp(-rf*T)*trapz(Integrand).*dK;
	DVFPercentError(k)  = (DVFRecoveredCall(k) - Call(k))/Call(k)*100;
    DVFCallIV(k) = fminsearch(@(v) BSIV(v,DVFRecoveredCall(k),S,K(k),rf,0,T,'C'), start);
    LNCallIV(k)  = fminsearch(@(v) BSIV(v,LNRecoveredCall(k), S,K(k),rf,0,T,'C'), start);
end;

%% Plot the market IV, and the IV from recovered DVF and LN prices
subplot(1,3,3)
plot(K,IVC,'kx-',K,DVFCallIV,'gx-',K,LNCallIV,'rx-')
legend('Market','Quadratic DVF','LN Mix')
title('Implied Volatilities')

%% Display the results
fprintf('RND method      Area under the curve \n')
fprintf('-------------------------------------\n')
fprintf('LN mixture       %8.4f \n',AreaUnderLN_RND) 
fprintf('Quadtratic DVF   %8.4f \n',AreaUnderDVF_RND) 
fprintf('-------------------------------------\n')
fprintf('\n')
fprintf('LN Mix and DVF recovered call prices and percent error \n')
fprintf('Strike   CallPrice   DVFCall  DVFError  LNMixCall  LNError \n')
fprintf('----------------------------------------------------------\n');
for k=1:length(Call)
    fprintf('%6.4f  %8.4f  %8.4f  %8.4f  %8.4f  %8.4f \n',...
        K(k),Call(k),DVFRecoveredCall(k),DVFPercentError(k),LNRecoveredCall(k),LNPercentError(k));
end
fprintf('----------------------------------------------------------\n');
fprintf('Mean Abs Percent Price Error %8.4f  %18.4f \n',...
    mean(abs(DVFPercentError)),mean(abs(LNPercentError)));
fprintf('----------------------------------------------------------\n');

